%
% (1) Computation of a gain-scheduled state-feedback observer for a missile LFT model
%
clear;
%
% sys_M and blk are loaded
%
% The first I/O of M(s) correspond to the interconnection with the structured model perturbation Delta
%
% The last input is the physical one.
% The last 3 outputs are Nz, q, alfa
%
load missile_LTI_open_loop.mat;
%
% Is the open loop LFT model well posed ?
%
nnn=sum(abs(blk(:,1)));
D11=sys_M.d(1:nnn,1:nnn);
muu=mu(D11,blk,'w');
fprintf('\n Is the open loop LFT model well posed ?\n');
if muu(1)<1      % mu upper bound
  fprintf('\n mu(D11) between %6.3e and %6.3e (well posed)\n',muu(2),muu(1));
elseif muu(2)>=1 % mu lower bound
  fprintf('\n mu(D11) between %6.3e and %6.3e (not well posed)\n',muu(2),muu(1));
else
  fprintf('\n mu(D11) between %6.3e and %6.3e (undecidable)\n',muu(2),muu(1));
end
pause_talk;
%
% Actuator model
%
omega_act=150;
num_act=[0 1; 1 0];
den_act=[1/omega_act/omega_act 2*0.7/omega_act 1];
%
N=sum(abs(blk(:,1)));
[a,b,c,d]=linmod('open_loop_missile');
%
% The state vector should be [q, alfa, actuator position, actuator rate]
%
T=c((N+1):(N+4),:);
Tinv=inv(T);
a=T*a*Tinv;
b=T*b;
c=c*Tinv;
%
% sys_M_complete:
% first I/O: for the interconnection with Delta.
% Last input = feedback input.
% Last outputs = state vector = [q, alfa, actuator position, actuator rate].
%
sys_M_complete=ss(a,b,c,d);
%
c=c(1:(N+2),:);
d=d(1:(N+2),:);
%
% sys_M_without_integrator:
% first I/O: for the interconnection with Delta.
% Last input = feedback input.
% Last outputs = q, alfa.
% state vector = [q, alfa, actuator position, actuator rate]
%
sys_M_without_integrator=ss(a,b,c,d);
%
c3=c(N+2,:); % alfa
nx=size(a,1);
[ny,nu]=size(d);
a=[a zeros(nx,1); c3 0];
b=[b; zeros(1,nu)];
c=[c zeros(ny,1); zeros(1,nx) 1];
d=[d; zeros(1,nu)];
sys_M_with_integrator=ss(a,b,c,d);
%
% sys_M_with_integrator:
% first I/O: for the interconnection with Delta.
% Last input = feedback input.
% Last outputs = q, alfa, Int alfa.
% state vector = [q, alfa, actuator position, actuator rate, Int alfa]
%

%
% puls = rough initial frequency gridding for mu analysis.
%
puls=[0 1000];
%
% sec_pol = [lambda r cxi] (degree of stability, maximum module, damping ratio)
%
% Choice of the design models: tab_delta(i,:) = value of the 2 parametric uncertainties.
%
% tol_P >= 0
%
% DESIGN OF THE STATE FEEDBACK GAIN
%
sec_pol_sf=[1 NaN 0.6]; % NaN means that r will be minimized on-line
%
tab_delta_sf=[-1 -1; -1 1; 1 -1; 1 1]*1.15;
tol_P_sf=1e-11;
%
% DESIGN OF THE OBSERVER GAIN
%
sec_pol_obs=[1 200 0.6];
tol_P_obs=1e-5;
tab_delta_obs=[-1 -1; -1 1; 1 -1; 1 1];
%
% multi-model design of the state-feedback gain
% validation with mu analysis
%
[K,OK]=calc_state_feedback_obs_lft_one_shot(sys_M_with_integrator,blk,sec_pol_sf,tab_delta_sf,tol_P_sf,0);
if ~OK return; end
%
% K1 = feedback on the missile states
% K2 = feedback on the integrator state
%
K1=K(:,1:(end-1));
K2=K(:,end);
%
fprintf('\n *** VALIDATION WITH MU ANALYSIS ***\n');
figure(1);
clf;
OK=calc_state_feedback_obs_lft_valid(K,sys_M_with_integrator,blk,sec_pol_sf,puls,0);
xlabel('rad/s');
ylabel('mu');
title('mu upper bound for the state-feedback gain');
pause_talk;
%
% multi-model design of the observer gain
% validation with mu analysis
%
[L,OK]=calc_state_feedback_obs_lft_one_shot(sys_M_without_integrator,blk,sec_pol_obs,tab_delta_obs,tol_P_obs,1);
if ~OK return; end
fprintf('\n *** VALIDATION WITH MU ANALYSIS ***\n');
figure(1);
clf;
OK=calc_state_feedback_obs_lft_valid(L,sys_M_without_integrator,blk,sec_pol_obs,puls,1);
xlabel('rad/s');
ylabel('mu');
title('mu upper bound for the observer gain');
%
save sf_obs_controller.mat K1 K2 L sys_M_with_integrator sys_M_without_integrator sys_M_complete blk...
sec_pol_sf tol_P_sf tab_delta_sf sec_pol_obs tol_P_obs tab_delta_obs puls;
