%
% Computation of the LFT controller on a gridding of models
%
% Validation on the closed loop LFT continuum a posteriori 
%
clear;
%
% The variable youla is loaded from youla.mat
%
load youla;
%
iterative=input2(...
'One-shot design on a small model gridding + validation on a large gridding (0) or iterative design on a large gridding (1): ',0,0:1);
%
% Griddings of models for the multi-model computation of Q(s) and its validation
%
if ~iterative
  tab_delta=[0 0; -1 -1; -1 1; 1 -1; 1 1];
  N=5;
  tab_delta_valid=calc_gridding([-1 1 N; -1 1 N]);  
else
  N=5;
  tab_delta=calc_gridding([-1 1 N; -1 1 N]);
  tab_delta_valid=tab_delta;
end
%
% List of the poles of the basis of Q(s)
%
% In the case of complex poles, just give one
%
list_poles=[-20 -20 -20 -20 -20 -20];
%
nu=length(youla.idxI2);
ny=length(youla.idxO2);
sys_Q0=ss([],[],[],zeros(nu,ny));
tab_sys_Q=gene_basis(list_poles,nu,ny);
Nbr_filters=length(tab_sys_Q);
%
% One shot or progressive design
%
one_shot=input2(' One shot (1) or progressive (0) design: ',1,0:1);
%
if one_shot
  tab_nbr_variables=Nbr_filters;
else
  shift_filters=max(ny*nu,4); % to be adjusted
  tab_nbr_variables=[];
  icpt=shift_filters;
  while 1
    if icpt>Nbr_filters break; end
    tab_nbr_variables=[tab_nbr_variables icpt];
    icpt=icpt+shift_filters;
  end
  if isempty(tab_nbr_variables)
    tab_nbr_variables=Nbr_filters;
  elseif tab_nbr_variables(end)~=Nbr_filters
    tab_nbr_variables=[tab_nbr_variables Nbr_filters];
  end
end
%
% Tuning parameters (mainly) for the cutting planes optimization solver
%
tuning=design_tuning;
%
% Definition of a fine frequency gridding for validation and H2 specifications
%
% The gridding is refined around the frequencies of the closed loop badly damped modes,
% defined as poles with a damping ratio between 0 and cxi
%
% N additional points around the frequency w of each closed loop badly damped mode
%
% More precisely, the points are defined as linspace((1-range)*w,(1+range)*w,N)
%
valid=[];
valid.puls=logspace2(1e-3,1e3,200);
%
valid.N=20;
valid.cxi=0.3;
valid.range=0.1;
%
% Definition of an initial rough frequency gridding for controller design:
% This gridding, whose size must be greater than Nbr_filters, will be refined.
%
puls_syn=linspace(0,1000,2*Nbr_filters);
puls_syn=sort(puls_syn);
%
% Definition of the specifications
%
% Exogenous inputs: alfa_ref (1) and delta_u (2)
% outputs: q (1), alfa (2), actuator position (3), actuator rate output (4), alfa-alfa_ref (5)
%
specif=[];
puls_specif=[0 1e3];
%
% SPECIFICATION N. 1
%
% delta_u --> alfa
%
specif{1}.input=2;
specif{1}.output=2;
specif{1}.list=[1 2];
specif{1}.template=[1 1];
specif{1}.obj=1;
specif{1}.lin_log_x=1;
specif{1}.lin_log_y=1;
specif{1}.interp=0;
specif{1}.models=1:size(tab_delta,1);
%
% SPECIFICATION N. 2
%
% delta_u --> actuator rate output
%
specif{2}.input=2;
specif{2}.output=4;
specif{2}.list=[1 2];
qqq=input2(' Constraint on the freq. response between delta u and the actuatuor rate output: ',150);
specif{2}.template=[qqq qqq];
specif{2}.obj=0;
specif{2}.lin_log_x=1;
specif{2}.lin_log_y=1;
specif{2}.interp=0;
specif{2}.models=1:size(tab_delta,1);
%
% performance visualization with the initial controller
%
fprintf('\n\n *** PERFORMANCE VISUALIZATION WITH THE INITIAL CONTROLLER ***\n\n');
youla_lft_validation_2(youla,sys_Q0,specif,puls_specif,Inf,valid,1,tab_delta_valid);
fprintf('\n figure 1 and following: performance with the initial controller\n');
pause_talk;
%
[sys_Q,alfa_opt,OK_design,puls_valid]=youla_lft_design_one_shot(youla,sys_Q0,tab_sys_Q,tab_delta,...
tab_nbr_variables,tuning,puls_syn,specif,puls_specif,list_poles,valid,iterative,1,100);
%
if ~OK_design return; end
%
% performance visualization with the optimal controller
%
fprintf('\n\n *** PERFORMANCE VISUALIZATION WITH THE OPTIMAL CONTROLLER ***\n\n');
youla_lft_validation_2(youla,sys_Q,specif,puls_specif,alfa_opt,valid,10,tab_delta_valid);
fprintf('\n figure 10 and following: performance with the optimal controller\n');
