%
% Convex design of the Youla parameter
%
clear;
%
close all;
%
load youla_file;
%
puls_valid=youla.puls_valid;
%
is_design_Q=input2(' Single computation of the Youla parameter (1) or study of the design trade-off (2): ',1,1:2);
%
% The Youla parameter will have the form: Q0(s) + \sum_i theta_i Qi(s)
% where filters Qi(s) are fixed while the theta_i are the optimisation variables.
%
% sys_Q0 is a state-space representation of the initial value Q0(s) of the Youla parameter.
%
% tab_sys_Q{i} is a state-space representation of Qi(s).
%
sys_Q0=sys_Q; % sys_Q comes from youla_file.mat and obs_state_feedback.mat
%
% List of the poles of the basis for the Youla parameter Q(s).
% In the case of complex poles, just give one pole.
% For example : list_poles=[-1 -2+sqrt(-1)*2];
%
if 1
  list_poles=-15*ones(1,3);
else
  list_poles=[5 8 12 15]*(-1+sqrt(-1))/sqrt(2);
end
%
N1=length(youla.idxI2); % N1 = number of outputs of Q(s) = number of plant  inputs.
N2=length(youla.idxO2); % N2 = number of  inputs of Q(s) = number of plant outputs.
%
tab_sys_Q=gene_basis(list_poles,N1,N2);
%
% a real pole gives N1*N2 filters.
% a pair of complex poles gives 2*N1*N2 filters.
%
Nbr_filters=length(tab_sys_Q); % Nbr_filters = number of filters Qi(s)
%
if is_design_Q==1
  one_shot=input2(' One-shot (1) or progressive (0) design: ',[],0:1);
else
  one_shot=1;
end
%
% In a one-shot design all filters will simultaneously be used.
% For a progressive design, i.e. a progressive introduction of an increasing number
% of filters, tab_nbr_variables can be an array.
% If e.g. tab_nbr_variables = [8 16 24], then the optimisation is performed w.r.t.
% the first 8 optimisation variables, then w.r.t. to the first 16 ones,
% and finally w.r.t. to all 24 ones.
%
if one_shot
  tab_nbr_variables=Nbr_filters;
else
  shift_filters=max(N1*N2,1); % to be adjusted
  tab_nbr_variables=[];
  icpt=shift_filters;
  while 1
    if icpt>Nbr_filters break; end
    tab_nbr_variables=[tab_nbr_variables icpt];
    icpt=icpt+shift_filters;
  end
  if isempty(tab_nbr_variables)
    tab_nbr_variables=Nbr_filters;
  elseif tab_nbr_variables(end)~=Nbr_filters
    tab_nbr_variables=[tab_nbr_variables Nbr_filters];
  end
end
%
% Definition of an initial ROUGH frequency gridding for controller design:
%
% zero point + nnn points between 1e-2 and 1e2 rad/s
%
% This gridding will be refined during the design.
%
% *** nnn must be greater than Nbr_filters ***
%
nnn=max(20,2*Nbr_filters);
puls_syn=[0 logspace2(1e-2,1e2,nnn)];
%
% Tuning parameters (mainly) for the cutting planes optimization solver
%
tuning=design_tuning;
%
% Let T(s) = [T1(s) T2(s); T3(s) T4(s)]. It is implicitly assumed that T4(s) = 0, so that the feedback
% interconnection of T(s) with Q(s) gives the closed loop transfer matrix T1(s) + T2(s) Q(s) T3(s).
% To check this point the vector theta of optimisation parameters is set to a random value.
% Q =  Q0(s) + \sum_i theta_i Qi(s) is computed, and then the frequency response F(jw) of the feedback
% interconnection of T(s) with Q(s). F(jw) is finally compared with the frequency response
% of T1(jw) + T2(jw) (Q0(jw) + \sum_i theta_i Qi(jw)) T3(jw). If T4(s) = 0 the error is zero.
%
is_affinity=input2('Do you want to check the affinity of the closed loop transfer matrix (0/1): ',1,0:1);
%
if is_affinity
  comp_freq_responses(puls_valid,[],youla,sys_Q0,tab_sys_Q,1,1,tuning);
  fprintf(' The error above should be very small\n');
  pause_talk;
end
%
if is_design_Q==1
%
% Single computation of the Youla parameter
%
  Gain_roll_off =input2(' Gain roll off: ',5);
  omega_roll_off=input2(' omega roll off (rad/s): ',20);
  [specif,puls_specif]=design_spec(Gain_roll_off,omega_roll_off);
%
  tic;
  [sys_Q,alfa_opt,OK_design]=design_convex...
  (youla,sys_Q0,tab_sys_Q,tab_nbr_variables,tuning,puls_syn,specif,puls_specif,puls_valid,0,1);
  fprintf('\n Elapsed time = %6.3f s\n',toc);
%
else
%
% Several computations of the Youla parameter
% to study the trade-off between design spec.
%
% The use of subgradients should save much computational time
%
  omega_roll_off=20;
  tab_gain_roll_off=1:2:15;
  tab_alfa_opt=[];
  subgradients=[];
  is_subgradients=input2(' Use of subgradients to save computational time (0/1): ',1,0:1);
  tic;
  for Gain_roll_off=tab_gain_roll_off
    [specif,puls_specif]=design_spec(Gain_roll_off,omega_roll_off);
    if ~is_subgradients subgradients=[]; end
    [sys_Q,alfa_opt,OK_design,subgradients]=design_convex...
    (youla,sys_Q0,tab_sys_Q,tab_nbr_variables,tuning,puls_syn,specif,puls_specif,puls_valid,0,1,subgradients);
    tab_alfa_opt=[tab_alfa_opt alfa_opt];
  end
  figure(1000);
  clf;
  plot(tab_gain_roll_off,tab_alfa_opt,tab_gain_roll_off,tab_alfa_opt,'*');    
  xlabel('Gain of the template');  
  ylabel('comfort criterion');
  fprintf('\n\n Elapsed time = %6.3f s\n',toc);
  return;
%
end
%
% The design failed if OK_design = 0
%
if ~OK_design return; end
%
% *** Visualisation of the performance of the initial controller ***
%
sys=feedback(youla.sys_T,sys_Q0,youla.idxI2,youla.idxO2,+1);
sys=sys(youla.idxO1,youla.idxI1);
freq_resp_valid=freqresp(sys,puls_valid);
options_valid=[];
options_valid.tol1=tuning.tol_constraint;
options_valid.tol2=0.01;
fprintf('\n *** INITIAL CLOSED LOOP ***\n');
validation(freq_resp_valid,puls_valid,specif,puls_specif,alfa_opt,options_valid,11);
%
% Direct computation of the closed loop as the interconnection of the open loop plant P(s) with
% the feedback controller K(s). The result should coincide with the one obtained on the first
% figures (done inside design_convex.m), where the closed loop was computed as the
% interconnection of T(s) with Q(s).
%
sys_Q_opt=sys_Q;
load open_loop_model;
load obs_state_feedback;
sys_Q=sys_Q_opt;
sys_K=comp_controller(sys_open_loop,feedback_inputs,feedback_outputs,K,L,sys_Q);
sys_open_loop=[sys_open_loop; 1 0 0 0; 0 1 0 0];
sys=feedback(sys_open_loop,sys_K,feedback_inputs,feedback_outputs,+1);
sys=sys(:,specif_inputs);
%
fprintf('\n\n Closed loop poles:\n');
damp(sys);
%
e=eig(sys);
if max(real(e))>=0
  fprintf('\n\n *** UNSTABLE CLOSED LOOP ***\n\n');
else
  freq_resp_valid=freqresp(sys,puls_valid);
  options_valid=[];
  options_valid.tol1=tuning.tol_constraint;
  options_valid.tol2=0.01;
  fprintf('\n *** OPTIMAL CLOSED LOOP ***\n');
  validation(freq_resp_valid,puls_valid,specif,puls_specif,alfa_opt,options_valid,21);
end
%
fprintf('\n\n Figures  1 to 3: optimal closed loop computed as the interconnection of T(s) with Q(s)\n');
fprintf(' Figures 11 to 13: initial closed loop computed as the interconnection of T(s) with Q0(s)\n');
fprintf(' Figures 21 to 23: optimal closed loop computed as the interconnection of P(s) with K(s)\n');
%
save controller.mat youla sys_Q sys_K specif puls_specif puls_valid sys_open_loop K L feedback_inputs feedback_outputs specif_inputs specif_outputs tuning alfa_opt;
