function [x,OK,lambda]=lp_qp_new(f,A,b,x0,lp_method,nbr_variables,is_gamma)
%
% call:  [x,OK,lambda]=lp_qp_new(f,A,b,x0,lp_method,nbr_variables,is_gamma)
%
% Minimisation  of f'x under LP constraints Ax <= b
%
% x0 is an initial guess which may not satisfy the LP constraints.
%
% Inside the routine, the LP solver can be chosen as:
% 1/ linprog.m
% 2/ quadprog.m
% 3/ simlp.m (the choice inside the routine)
%
% Use of the input argument lp_method, only if linprog.m or quadprog.m are used:
% lp_method = 1 --> use of linprog.m with the large-scale option set to off
%           = 2 --> use of linprog.m with the large-scale option set to on
%                   (this method doesn't take into account the initial guess x0)
%
% nbr_variables is the true number of optimization parameters.
% Let x1 be the first nbr_variables scalar optimisation parameters,
% and x2 be the other scalar optimisation parameters.
% x2 is fixed to its initial value (as defined by its corresponding part in x0),
% while x1 is free.
%
% OK > 0 means that the solution x is reliable.
%
% lambda is the vector of multipliers associated to the inequalities.
%
% is_gamma = 0 --> the last optimisation parameter is gamma or sigma.
%          = 1 --> The last optimisation parameters are gamma and sigma.
%
if nargin~=7 error('LP_QP_NEW: 7 input arguments'); end
%
n=length(x0);
if is_gamma
  idx1=[1:nbr_variables n-1 n];     
  idx2=(nbr_variables+1):(n-2);
else
  idx1=[1:nbr_variables n];     
  idx2=(nbr_variables+1):(n-1);
end
if ~isempty(A)
  A1=A(:,idx1);
  b1=b-A(:,idx2)*x0(idx2);
else
  A1=[];
  b1=[];
end
x01=x0(idx1);
x02=x0(idx2);
f1=f(idx1);
%
if lp_method==1
  options=optimset('LargeScale','off','Display','off');
else
  options=optimset('LargeScale','on','Display','off');
end
%
% lp_solver_choice = 1 --> Use of linprog.m
%                    2 --> Use of quadprog.m
%                    3 --> Use of simlp.m
%
lp_solver_choice=3;
%
warning off;
if lp_solver_choice==1
  [x,fval,exitflag,output,lambda]=linprog(f1,A1,b1,[],[],[],[],x01,options);
  OK=(exitflag>0);
  lambda=lambda.ineqlin;  
elseif lp_solver_choice==2
  Q1=zeros(length(f1),length(f1));
  [x,fval,exitflag,output,lambda]=quadprog(Q1,f1,A1,b1,[],[],[],[],x01,options);
  OK=(exitflag>0);
  lambda=lambda.ineqlin;
else
  [x,lambda,how]=simlp(f1,A1,b1,[],[],x01);
  OK=strcmp(how,'ok');
  fval=[];
  exitflag=[];
  output=[];
end
warning on;
%
if length(x)~=length(x01)
  x=[];
  lambda=[];
  OK=0;
  return;
end
%
if is_gamma
  x=[x(1:nbr_variables); x02; x([end-1 end])];
else
  x=[x(1:nbr_variables); x02; x(end)];
end
